@@ -3,10 +3,19 @@
 
 function get_deps() {
 
-	TSS_VERSION=2.4.0
-	ABRMD_VERSION=2.3.1
+	TSS_VERSION=${TPM2_TSS_VERSION:-3.0.1}
+	ABRMD_VERSION=2.3.3
 
 	echo "pwd starting: `pwd`"
+	if [ "$TSS_VERSION" = "2.4.0" ]; then
+		if [ "$DOCKER_IMAGE" = "fedora-30" ]; then
+			yum -y install libgcrypt-devel
+		elif [ "$DOCKER_IMAGE" = "opensuse-leap" ]; then
+			zypper -n in libgcrypt-devel
+		elif [ "$DOCKER_IMAGE" = "ubuntu-20.04" -o "$DOCKER_IMAGE" = "ubuntu-18.04" ]; then
+			apt-get -y install libgcrypt20-dev
+		fi
+	fi
 	pushd "$1"
 	echo "pwd clone tss: `pwd`"
 	if [ ! -d tpm2-tss ]; then
