@@ -1,16 +1,16 @@
-#!/bin/bash
 
 set -e
 source helpers.sh
 
 start_up
 
-setup_fapi
+CRYPTO_PROFILE="RSA"
+setup_fapi $CRYPTO_PROFILE
 
 PATH=${BUILDDIR}/tools/fapi:$PATH
 
 function cleanup {
-    tss2_delete --path /
+    tss2 delete --path=/
     shut_down
 }
 
@@ -18,11 +18,11 @@ trap cleanup EXIT
 
 OUTPUT_FILE="$TEMP_DIR/output.file"
 
-tss2_provision
+tss2 provision
 
 expect <<EOF
 # Try with wrong size value
-spawn tss2_getrandom --numBytes a --data $OUTPUT_FILE --force
+spawn tss2 getrandom --numBytes=a --data=$OUTPUT_FILE --force
 set ret [wait]
 if {[lindex \$ret 2] || [lindex \$ret 3] != 1} {
     Command has not failed as expected\n"
@@ -32,7 +32,7 @@ EOF
 
 expect <<EOF
 # Try with missing output
-spawn tss2_getrandom --numBytes 20
+spawn tss2 getrandom --numBytes=20
 set ret [wait]
 if {[lindex \$ret 2] || [lindex \$ret 3] != 1} {
     Command has not failed as expected\n"
@@ -42,7 +42,7 @@ EOF
 
 expect <<EOF
 # Try with missing numBytes
-spawn tss2_getrandom --data $OUTPUT_FILE --force
+spawn tss2 getrandom --data=$OUTPUT_FILE --force
 set ret [wait]
 if {[lindex \$ret 2] || [lindex \$ret 3] != 1} {
     Command has not failed as expected\n"
@@ -50,7 +50,8 @@ if {[lindex \$ret 2] || [lindex \$ret 3] != 1} {
 }
 EOF
 
-tss2_getrandom --numBytes 4 --data $OUTPUT_FILE --force
+tss2 getrandom --numBytes=4 --data=$OUTPUT_FILE --force
 
+tss2 getrandom --numBytes=4 --hex --data=$OUTPUT_FILE --force
 
 exit 0
