@@ -8,7 +8,7 @@ cleanup() {
 
     # Evict persistent handles, we want them to always succeed and never trip
     # the onerror trap.
-    tpm2_evictcontrol -Q -C o -c 0x81010009 2>/dev/null || true
+    tpm2 evictcontrol -Q -C o -c 0x81010009 2>/dev/null || true
 
     if [ "$1" != "no-shut-down" ]; then
         shut_down
@@ -22,11 +22,28 @@ cleanup "no-shut-down"
 
 echo 12345678 > secret.data
 
-tpm2_createek -Q -c 0x81010009 -G rsa -u ek.pub
+tpm2 createek -Q -c 0x81010009 -G rsa -u ek.pub
 
-tpm2_createak -C 0x81010009 -c ak.ctx -G rsa -g sha256 -s rsassa -u ak.pub \
+tpm2 createak -C 0x81010009 -c ak.ctx -G rsa -g sha256 -s rsassa -u ak.pub \
 -n ak.name -p akpass> ak.out
 
+file_size=`ls -l ak.name | awk {'print $5'}`
+loaded_key_name=`cat ak.name | xxd -p -c $file_size` # Use -c in xxd so there is no line wrapping
+
+tpm2 readpublic -c 0x81010009 -o ek.pem -f pem -Q
+
+echo "12345678" | tpm2 makecredential -Q -u ek.pem -s - -n $loaded_key_name \
+-o mkcred.out -G rsa
+
+# Test the secret data matches after credential activation process
+tpm2 startauthsession --policy-session -S session.ctx
+tpm2 policysecret -S session.ctx -c e
+tpm2 activatecredential -Q -c ak.ctx -C 0x81010009 -i mkcred.out \
+-o actcred.out -p akpass -P"session:session.ctx"
+tpm2 flushcontext session.ctx
+
+diff actcred.out secret.data
+
 # Capture the yaml output and verify that its the same as the name output
 loaded_key_name_yaml=`python << pyscript
 from __future__ import print_function
@@ -38,20 +55,6 @@ with open('ak.out', 'r') as f:
     print(doc['loaded-key']['name'])
 pyscript`
 
-# Use -c in xxd so there is no line wrapping
-file_size=`ls -l ak.name | awk {'print $5'}`
-loaded_key_name=`cat ak.name | xxd -p -c $file_size`
-
 test "$loaded_key_name_yaml" == "$loaded_key_name"
 
-tpm2_makecredential -Q -e ek.pub -s secret.data -n $loaded_key_name \
--o mkcred.out
-
-TPM2_RH_ENDORSEMENT=0x4000000B
-tpm2_startauthsession --policy-session -S session.ctx
-tpm2_policysecret -S session.ctx -c $TPM2_RH_ENDORSEMENT
-tpm2_activatecredential -Q -c ak.ctx -C 0x81010009 -i mkcred.out \
--o actcred.out -p akpass -P"session:session.ctx"
-tpm2_flushcontext session.ctx
-
 exit 0
