@@ -16,20 +16,20 @@ start_up
 
 cleanup "no-shut-down"
 
-tpm2_clear -Q
+tpm2 clear -Q
 
-tpm2_createprimary -C o -c primary.ctx -Q
+tpm2 createprimary -C o -c primary.ctx -Q
 
-tpm2_create -G rsa -u signing_key.pub -r signing_key.priv -C primary.ctx \
+tpm2 create -G rsa -u signing_key.pub -r signing_key.priv -C primary.ctx \
 -c signing_key.ctx -Q
 
-tpm2_readpublic -c signing_key.ctx -f pem -o sslpub.pem -Q
+tpm2 readpublic -c signing_key.ctx -f pem -o sslpub.pem -Q
 
-tpm2_nvdefine -s 32 -a "authread|authwrite" 1
+tpm2 nvdefine -s 32 -a "authread|authwrite" 1
 
-dd if=/dev/urandom bs=1 count=32 status=none| tpm2_nvwrite 1 -i-
+dd if=/dev/urandom bs=1 count=32 status=none| tpm2 nvwrite 1 -i-
 
-tpm2_nvcertify -C signing_key.ctx -g sha256 -f plain -s rsassa \
+tpm2 nvcertify -C signing_key.ctx -g sha256 -f plain -s rsassa \
 -o signature.bin --attestation attestation.bin --size 32 1
 
 openssl dgst -verify sslpub.pem -keyform pem -sha256 -signature signature.bin \
@@ -40,7 +40,7 @@ attestation.bin
 #
 dd if=/dev/urandom of=qual.dat bs=1 count=32
 
-tpm2_nvcertify -C signing_key.ctx -g sha256 -f plain -s rsassa \
+tpm2 nvcertify -C signing_key.ctx -g sha256 -f plain -s rsassa \
 -o signature.bin --attestation attestation.bin --size 32 -q qual.dat 1
 
 openssl dgst -verify sslpub.pem -keyform pem -sha256 -signature signature.bin \
