@@ -12,6 +12,7 @@
 #include "tpm2_eventlog.h"
 
 #define TCG_DIGEST2_SHA1_SIZE (sizeof(TCG_DIGEST2) + TPM2_SHA_DIGEST_SIZE)
+#define TCG_DIGEST2_SHA256_SIZE (sizeof(TCG_DIGEST2) + TPM2_SHA256_DIGEST_SIZE)
 
 static bool foreach_digest2_test_callback(TCG_DIGEST2 const *digest, size_t size, void *data){
 
@@ -24,16 +25,18 @@ static bool foreach_digest2_test_callback(TCG_DIGEST2 const *digest, size_t size
 static void test_foreach_digest2_null(void **state){
 
     (void)state;
+    tpm2_eventlog_context ctx = {0};
 
-    assert_false(foreach_digest2(NULL, 0, sizeof(TCG_DIGEST2), NULL, NULL));
+    assert_false(foreach_digest2(&ctx, 0, NULL, 0, sizeof(TCG_DIGEST2)));
 }
 static void test_foreach_digest2_size(void **state) {
 
     (void)state;
     uint8_t buf [sizeof(TCG_DIGEST2) - 1] = { 0, };
     TCG_DIGEST2 *digest = (TCG_DIGEST2*)buf;
+    tpm2_eventlog_context ctx = { .digest2_cb = foreach_digest2_test_callback };
 
-    assert_false(foreach_digest2(digest, 1, sizeof(TCG_DIGEST2) - 1, foreach_digest2_test_callback, NULL));
+    assert_false(foreach_digest2(&ctx, 0, digest, 1, sizeof(TCG_DIGEST2) - 1));
 }
 static void test_foreach_digest2(void **state) {
 
@@ -42,15 +45,56 @@ static void test_foreach_digest2(void **state) {
     TCG_DIGEST2* digest = (TCG_DIGEST2*)buf;
 
     will_return(foreach_digest2_test_callback, true);
-    assert_true(foreach_digest2(digest, 1, TCG_DIGEST2_SHA1_SIZE, foreach_digest2_test_callback, NULL));
+
+    tpm2_eventlog_context ctx = { .digest2_cb = foreach_digest2_test_callback };
+    assert_true(foreach_digest2(&ctx, 0, digest, 1, TCG_DIGEST2_SHA1_SIZE));
 }
 static void test_foreach_digest2_cbnull(void **state){
 
     (void)state;
-    uint8_t buf [TCG_DIGEST2_SHA1_SIZE] = { 0, };
+    uint8_t buf [TCG_DIGEST2_SHA1_SIZE] = {0};
     TCG_DIGEST2* digest = (TCG_DIGEST2*)buf;
 
-    assert_true(foreach_digest2(digest, 1, TCG_DIGEST2_SHA1_SIZE, NULL, NULL));
+    tpm2_eventlog_context ctx = {0};
+    assert_true(foreach_digest2(&ctx, 0, digest, 1, TCG_DIGEST2_SHA1_SIZE));
+}
+static void test_sha1(void **state){
+
+    (void)state;
+    uint8_t buf [TCG_DIGEST2_SHA1_SIZE] = { 0, };
+    const uint8_t sha1sum[] = {
+        0x31,0x19,0x5d,0x69,0x35,0x16,0x3c,0x79,0xa9,0x67,
+        0x22,0xba,0x7d,0x4b,0x11,0x35,0x24,0x89,0xf4,0x8b,
+    };
+    const int pcr_index = 3;
+
+    TCG_DIGEST2 * digest = (TCG_DIGEST2*) buf;
+    digest->AlgorithmId = TPM2_ALG_SHA1,
+    memcpy(digest->Digest, "the magic words are:", TPM2_SHA1_DIGEST_SIZE);
+
+    tpm2_eventlog_context ctx = {0};
+    assert_true(foreach_digest2(&ctx, pcr_index, digest, 1, TCG_DIGEST2_SHA1_SIZE));
+    assert_memory_equal(ctx.sha1_pcrs[pcr_index], sha1sum, sizeof(sha1sum));
+}
+static void test_sha256(void **state){
+
+    (void)state;
+    uint8_t buf [TCG_DIGEST2_SHA256_SIZE] = {0};
+    const uint8_t sha256sum[] = {
+        0x51,0xea,0x4e,0xa4,0x98,0xaa,0xe2,0x52,
+        0xf4,0xe7,0xff,0x4b,0x13,0xb6,0x3f,0xe5,
+        0xb5,0x7a,0xf8,0x21,0xa4,0x84,0x4e,0xe2,
+        0x6f,0xd8,0xdd,0x25,0xa4,0x2b,0x33,0x23,
+    };
+    const int pcr_index = 3;
+
+    TCG_DIGEST2 * digest = (TCG_DIGEST2*) buf;
+    digest->AlgorithmId = TPM2_ALG_SHA256,
+    memcpy(digest->Digest, "The Magic Words are Squeamish Ossifrage, for RSA-129 (from 1977)", TPM2_SHA256_DIGEST_SIZE);
+
+    tpm2_eventlog_context ctx = {0};
+    assert_true(foreach_digest2(&ctx, pcr_index, digest, 1, TCG_DIGEST2_SHA256_SIZE));
+    assert_memory_equal(ctx.sha256_pcrs[pcr_index], sha256sum, sizeof(sha256sum));
 }
 static void test_foreach_digest2_cbfail(void **state){
 
@@ -59,7 +103,9 @@ static void test_foreach_digest2_cbfail(void **state){
     TCG_DIGEST2* digest = (TCG_DIGEST2*)buf;
 
     will_return(foreach_digest2_test_callback, false);
-    assert_false(foreach_digest2(digest, 1, TCG_DIGEST2_SHA1_SIZE, foreach_digest2_test_callback, NULL));
+
+    tpm2_eventlog_context ctx = { .digest2_cb = foreach_digest2_test_callback };
+    assert_false(foreach_digest2(&ctx, 0, digest, 1, TCG_DIGEST2_SHA1_SIZE));
 }
 static void test_digest2_accumulator_callback(void **state) {
 
@@ -146,12 +192,6 @@ static void test_parse_event2_badeventbuf(void **state){
 
     assert_false(parse_event2(eventhdr, sizeof(buf), &size_event, &size_digest));
 }
-static void test_foreach_event2_null(void **state){
-
-    (void)state;
-
-    assert_false(foreach_event2(NULL, 0, NULL, NULL, NULL, NULL));
-}
 static void test_foreach_event2(void **state){
 
     (void)state;
@@ -167,9 +207,13 @@ static void test_foreach_event2(void **state){
     will_return(test_event2hdr_callback, true);
     will_return(foreach_digest2_test_callback, true);
     will_return(test_event2_callback, true);
-    assert_true(foreach_event2(eventhdr, sizeof(buf), test_event2hdr_callback,
-                               foreach_digest2_test_callback,
-                               test_event2_callback, NULL));
+
+    tpm2_eventlog_context ctx = {
+        .digest2_cb = foreach_digest2_test_callback,
+        .event2_cb = test_event2_callback,
+        .event2hdr_cb = test_event2hdr_callback,
+    };
+    assert_true(foreach_event2(&ctx, eventhdr, sizeof(buf)));
 }
 static void test_foreach_event2_event2hdr_fail(void **state){
 
@@ -184,8 +228,11 @@ static void test_foreach_event2_event2hdr_fail(void **state){
     event->EventSize = 1;
 
     will_return(test_event2hdr_callback, false);
-    assert_false(foreach_event2(eventhdr, sizeof(buf), test_event2hdr_callback,
-                                NULL, NULL, NULL));
+
+    tpm2_eventlog_context ctx = {
+        .event2hdr_cb = test_event2hdr_callback,
+    };
+    assert_false(foreach_event2(&ctx, eventhdr, sizeof(buf)));
 }
 static void test_foreach_event2_digest2_fail(void **state){
 
@@ -201,9 +248,12 @@ static void test_foreach_event2_digest2_fail(void **state){
 
     will_return(test_event2hdr_callback, true);
     will_return(foreach_digest2_test_callback, false);
-    assert_false(foreach_event2(eventhdr, sizeof(buf),
-                                test_event2hdr_callback,
-                                foreach_digest2_test_callback, NULL, NULL));
+
+    tpm2_eventlog_context ctx = {
+        .digest2_cb = foreach_digest2_test_callback,
+        .event2hdr_cb = test_event2hdr_callback,
+    };
+    assert_false(foreach_event2(&ctx, eventhdr, sizeof(buf)));
 }
 static void test_foreach_event2_parse_event2body_fail(void **state){
 
@@ -221,9 +271,12 @@ static void test_foreach_event2_parse_event2body_fail(void **state){
 
     will_return(test_event2hdr_callback, true);
     will_return(foreach_digest2_test_callback, true);
-    assert_false(foreach_event2(eventhdr, sizeof(buf),
-                                test_event2hdr_callback,
-                                foreach_digest2_test_callback, NULL, NULL));
+
+    tpm2_eventlog_context ctx = {
+        .digest2_cb = foreach_digest2_test_callback,
+        .event2hdr_cb = test_event2hdr_callback,
+    };
+    assert_false(foreach_event2(&ctx, eventhdr, sizeof(buf)));
 }
 static void test_foreach_event2_event2body_fail(void **state){
 
@@ -240,10 +293,13 @@ static void test_foreach_event2_event2body_fail(void **state){
     will_return(test_event2hdr_callback, true);
     will_return(foreach_digest2_test_callback, true);
     will_return(test_event2_callback, false);
-    assert_false(foreach_event2(eventhdr, sizeof(buf),
-                                test_event2hdr_callback,
-                                foreach_digest2_test_callback,
-                                test_event2_callback, NULL));
+
+    tpm2_eventlog_context ctx = {
+        .event2hdr_cb = test_event2hdr_callback,
+        .digest2_cb = foreach_digest2_test_callback,
+        .event2_cb = test_event2_callback,
+    };
+    assert_false(foreach_event2(&ctx, eventhdr, sizeof(buf)));
 }
 static void test_parse_event2body_uefivar_badsize(void **state){
 
@@ -266,7 +322,7 @@ static void test_parse_event2body_uefivar_badlength(void **state){
 
     assert_false(parse_event2body(event, EV_EFI_VARIABLE_DRIVER_CONFIG));
 }
-static void test_parse_event2body_postcode_badlength(void **state){
+static void test_parse_event2body_firmware_blob_badlength(void **state){
 
     (void)state;
 
@@ -451,13 +507,14 @@ int main(void) {
         cmocka_unit_test(test_foreach_digest2),
         cmocka_unit_test(test_foreach_digest2_cbfail),
         cmocka_unit_test(test_foreach_digest2_cbnull),
+        cmocka_unit_test(test_sha1),
+        cmocka_unit_test(test_sha256),
         cmocka_unit_test(test_digest2_accumulator_callback),
         cmocka_unit_test(test_digest2_accumulator_callback_null),
         cmocka_unit_test(test_parse_event2_badhdr),
         cmocka_unit_test(test_parse_event2_baddigest),
         cmocka_unit_test(test_parse_event2_badeventsize),
         cmocka_unit_test(test_parse_event2_badeventbuf),
-        cmocka_unit_test(test_foreach_event2_null),
         cmocka_unit_test(test_foreach_event2),
         cmocka_unit_test(test_foreach_event2_event2hdr_fail),
         cmocka_unit_test(test_foreach_event2_event2body_fail),
@@ -465,7 +522,7 @@ int main(void) {
         cmocka_unit_test(test_foreach_event2_parse_event2body_fail),
         cmocka_unit_test(test_parse_event2body_uefivar_badsize),
         cmocka_unit_test(test_parse_event2body_uefivar_badlength),
-        cmocka_unit_test(test_parse_event2body_postcode_badlength),
+        cmocka_unit_test(test_parse_event2body_firmware_blob_badlength),
         cmocka_unit_test(test_specid_event_nohdr),
         cmocka_unit_test(test_specid_event_badeventtype),
         cmocka_unit_test(test_specid_event_badpcrindex),
