@@ -6,9 +6,6 @@
 #include <string.h>
 #include "tools/fapi/tss2_template.h"
 
-/* needed by tpm2_util and tpm2_option functions */
-bool output_enabled = false;
-
 /* Context struct used to store passed commandline parameters */
 static struct cxt {
     char *info;
@@ -29,7 +26,7 @@ static bool on_option(char key, char *value) {
 }
 
 /* Define possible commandline parameters */
-bool tss2_tool_onstart(tpm2_options **opts) {
+static bool tss2_tool_onstart(tpm2_options **opts) {
     struct option topts[] = {
         {"force"   , no_argument      , NULL, 'f'},
         /* output file */
@@ -40,7 +37,7 @@ bool tss2_tool_onstart(tpm2_options **opts) {
 }
 
 /* Execute specific tool */
-int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
+static int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
     /* Check availability of required parameters */
     if (!ctx.info) {
         fprintf (stderr, "info parameter is missing, pass --info\n");
@@ -58,7 +55,6 @@ int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
     /* Write returned data to file(s) */
     r = open_write_and_close (ctx.info, ctx.overwrite, info, strlen(info));
     if (r) {
-        LOG_PERR ("open_write_and_close", r);
         Fapi_Free (info);
         return 1;
     }
@@ -66,3 +62,5 @@ int tss2_tool_onrun (FAPI_CONTEXT *fctx) {
     Fapi_Free (info);
     return 0;
 }
+
+TSS2_TOOL_REGISTER("getinfo", tss2_tool_onstart, tss2_tool_onrun, NULL)
