@@ -7,6 +7,7 @@
 #include "files.h"
 #include "log.h"
 #include "tpm2.h"
+#include "tpm2_tool.h"
 #include "tpm2_alg_util.h"
 #include "tpm2_convert.h"
 #include "tpm2_options.h"
@@ -42,7 +43,7 @@ struct tpm_certifycreation_ctx {
 };
 
 static tpm_certifycreation_ctx ctx = {
-        .halg = TPM2_ALG_SHA1,
+        .halg = TPM2_ALG_NULL,
         .sig_scheme = TPM2_ALG_NULL
 };
 
@@ -124,7 +125,7 @@ on_option_out:
     return result;
 }
 
-bool tpm2_tool_onstart(tpm2_options **opts) {
+static bool tpm2_tool_onstart(tpm2_options **opts) {
 
     static const struct option topts[] = {
       { "signingkey-context",   required_argument, NULL, 'C' },
@@ -219,7 +220,7 @@ static tool_rc process_certifycreation_input(ESYS_CONTEXT *ectx,
      * Validate chosen scheme is allowed for key type
      */
     rc = tpm2_alg_util_get_signature_scheme(ectx,
-        ctx.signing_key.object.tr_handle, ctx.halg, ctx.sig_scheme, in_scheme);
+        ctx.signing_key.object.tr_handle, &ctx.halg, ctx.sig_scheme, in_scheme);
     if (rc != tool_rc_success) {
         LOG_ERR("bad signature scheme for key type!");
         return rc;
@@ -269,7 +270,7 @@ static tool_rc process_certifycreation_output(TPMT_SIGNATURE *signature,
     return tool_rc_success;
 }
 
-tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
+static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
 
     UNUSED(flags);
 
@@ -324,7 +325,10 @@ tpm2_tool_onrun_out:
     return rc;
 }
 
-tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
+static tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
     UNUSED(ectx);
     return tpm2_session_close(&ctx.signing_key.object.session);
 }
+
+// Register this tool with tpm2_tool.c
+TPM2_TOOL_REGISTER("certifycreation", tpm2_tool_onstart, tpm2_tool_onrun, tpm2_tool_onstop, NULL)
