@@ -6,6 +6,7 @@
 #include "files.h"
 #include "log.h"
 #include "tpm2.h"
+#include "tpm2_tool.h"
 #include "tpm2_alg_util.h"
 #include "tpm2_options.h"
 
@@ -41,6 +42,7 @@ struct tpm_duplicate_ctx {
         UINT16 s :1;
     } flags;
 
+    char *cp_hash_path;
 };
 
 static tpm_duplicate_ctx ctx = {
@@ -51,9 +53,25 @@ static tool_rc do_duplicate(ESYS_CONTEXT *ectx, TPM2B_DATA *in_key,
         TPMT_SYM_DEF_OBJECT *sym_alg, TPM2B_DATA **out_key,
         TPM2B_PRIVATE **duplicate, TPM2B_ENCRYPTED_SECRET **encrypted_seed) {
 
+    if (ctx.cp_hash_path) {
+        TPM2B_DIGEST cp_hash = { .size = 0 };
+        tool_rc rc = tpm2_duplicate(ectx, &ctx.duplicable_key.object,
+            &ctx.new_parent_key.object, in_key, sym_alg, out_key, duplicate,
+            encrypted_seed, &cp_hash);
+        if (rc != tool_rc_success) {
+            return rc;
+        }
+
+        bool result = files_save_digest(&cp_hash, ctx.cp_hash_path);
+        if (!result) {
+            rc = tool_rc_general_error;
+        }
+        return rc;
+    }
+
     return tpm2_duplicate(ectx, &ctx.duplicable_key.object,
-            ctx.new_parent_key.object.tr_handle, in_key, sym_alg, out_key,
-            duplicate, encrypted_seed);
+            &ctx.new_parent_key.object, in_key, sym_alg, out_key, duplicate,
+            encrypted_seed, NULL);
 }
 
 static bool on_option(char key, char *value) {
@@ -93,6 +111,9 @@ static bool on_option(char key, char *value) {
         ctx.enc_seed_out = value;
         ctx.flags.s = 1;
         break;
+    case 0:
+        ctx.cp_hash_path = value;
+        break;
     default:
         LOG_ERR("Invalid option");
         return false;
@@ -101,7 +122,7 @@ static bool on_option(char key, char *value) {
     return true;
 }
 
-bool tpm2_tool_onstart(tpm2_options **opts) {
+static bool tpm2_tool_onstart(tpm2_options **opts) {
 
     const struct option topts[] = {
       { "auth",              required_argument, NULL, 'p'},
@@ -112,6 +133,7 @@ bool tpm2_tool_onstart(tpm2_options **opts) {
       { "encrypted-seed",    required_argument, NULL, 's'},
       { "parent-context",    required_argument, NULL, 'C'},
       { "key-context",       required_argument, NULL, 'c'},
+      { "cphash",            required_argument, NULL,  0 },
     };
 
     *opts = tpm2_options_new("p:G:i:C:o:s:r:c:", ARRAY_LEN(topts), topts,
@@ -202,7 +224,7 @@ static bool set_key_algorithm(TPMI_ALG_PUBLIC alg, TPMT_SYM_DEF_OBJECT * obj) {
     return result;
 }
 
-tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
+static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
     UNUSED(flags);
 
     tool_rc rc = tool_rc_general_error;
@@ -252,7 +274,7 @@ tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
 
     rc = do_duplicate(ectx, ctx.flags.i ? &in_key : NULL, &sym_alg,
             ctx.flags.o ? &out_key : NULL, &duplicate, &out_sym_seed);
-    if (rc != tool_rc_success) {
+    if (rc != tool_rc_success || ctx.cp_hash_path) {
         return rc;
     }
 
@@ -299,8 +321,11 @@ out:
     return rc;
 }
 
-tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
+static tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
     UNUSED(ectx);
 
     return tpm2_session_close(&ctx.duplicable_key.object.session);
 }
+
+// Register this tool with tpm2_tool.c
+TPM2_TOOL_REGISTER("duplicate", tpm2_tool_onstart, tpm2_tool_onrun, tpm2_tool_onstop, NULL)
