@@ -295,7 +295,8 @@ static void dump_tpm_properties_fixed(TPMS_TAGGED_PROPERTY properties[],
             break;
         case TPM2_PT_REVISION:
             tpm2_tool_output("TPM2_PT_REVISION:\n"
-                    "  value: %.2f\n", (float )value / 100);
+                    "  raw: 0x%X\n"
+                    "  value: %.2f\n", value, (float )value / 100);
             break;
         case TPM2_PT_DAY_OF_YEAR:
             tpm2_tool_output("TPM2_PT_DAY_OF_YEAR:\n"
@@ -806,7 +807,7 @@ static bool on_arg(int argc, char *argv[]) {
     return true;
 }
 
-bool tpm2_tool_onstart(tpm2_options **opts) {
+static bool tpm2_tool_onstart(tpm2_options **opts) {
 
     const struct option topts[] = { { "list", no_argument, NULL, 'l' },
 
@@ -818,7 +819,7 @@ bool tpm2_tool_onstart(tpm2_options **opts) {
     return *opts != NULL;
 }
 
-tool_rc tpm2_tool_onrun(ESYS_CONTEXT *context, tpm2_option_flags flags) {
+static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *context, tpm2_option_flags flags) {
 
     UNUSED(flags);
 
@@ -850,3 +851,6 @@ tool_rc tpm2_tool_onrun(ESYS_CONTEXT *context, tpm2_option_flags flags) {
     free(capability_data);
     return result ? tool_rc_success : tool_rc_general_error;
 }
+
+// Register this tool with tpm2_tool.c
+TPM2_TOOL_REGISTER("getcap", tpm2_tool_onstart, tpm2_tool_onrun, NULL, NULL)
