@@ -26,6 +26,7 @@ struct tpm_hmac_ctx {
     char *ticket_path;
     TPMI_ALG_HASH halg;
     bool hex;
+    char *cp_hash_path;
 };
 
 static tpm_hmac_ctx ctx;
@@ -54,11 +55,33 @@ static tool_rc tpm_hmac_file(ESYS_CONTEXT *ectx, TPM2B_DIGEST **result,
             LOG_ERR("Error reading input file!");
             return tool_rc_general_error;
         }
+
+        if (ctx.cp_hash_path) {
+            LOG_WARN("Exiting without performing HMAC when calculating cpHash");
+            TPM2B_DIGEST cp_hash = { .size = 0 };
+            tool_rc rc = tpm2_hmac(ectx, &ctx.hmac_key.object, ctx.halg,
+            &buffer, result, &cp_hash);
+            if (rc != tool_rc_success) {
+                return rc;
+            }
+
+            bool result = files_save_digest(&cp_hash, ctx.cp_hash_path);
+            if (!result) {
+                rc = tool_rc_general_error;
+            }
+            return rc;
+        }
         /*
          * hash algorithm specified in the key's scheme is used as the
          * hash algorithm for the HMAC
          */
-        return tpm2_hmac(ectx, &ctx.hmac_key.object, ctx.halg, &buffer, result);
+        return tpm2_hmac(ectx, &ctx.hmac_key.object, ctx.halg, &buffer, result,
+        NULL);
+    }
+
+    if (ctx.cp_hash_path) {
+        LOG_ERR("Cannot calculate cpHash for buffers requiring HMAC sequence.");
+        return tool_rc_general_error;
     }
 
     ESYS_TR sequence_handle;
@@ -139,7 +162,7 @@ static tool_rc do_hmac_and_output(ESYS_CONTEXT *ectx) {
     FILE *out = stdout;
 
     tool_rc rc = tpm_hmac_file(ectx, &hmac_out, &validation);
-    if (rc != tool_rc_success) {
+    if (rc != tool_rc_success || ctx.cp_hash_path) {
         goto out;
     }
 
@@ -212,6 +235,9 @@ static bool on_option(char key, char *value) {
     case 0:
         ctx.hex = true;
         break;
+    case 1:
+        ctx.cp_hash_path = value;
+        break;
         /* no default */
     }
 
@@ -235,7 +261,7 @@ static bool on_args(int argc, char **argv) {
     return true;
 }
 
-bool tpm2_tool_onstart(tpm2_options **opts) {
+static bool tpm2_tool_onstart(tpm2_options **opts) {
 
     const struct option topts[] = {
         { "key-context",    required_argument, NULL, 'c' },
@@ -244,6 +270,7 @@ bool tpm2_tool_onstart(tpm2_options **opts) {
         { "hash-algorithm", required_argument, NULL, 'g' },
         { "ticket",         required_argument, NULL, 't' },
         { "hex",            no_argument,       NULL,  0  },
+        { "cphash",         required_argument, NULL,  1  },
     };
 
     ctx.input = stdin;
@@ -260,7 +287,7 @@ static tool_rc readpub(ESYS_CONTEXT *ectx, ESYS_TR handle,
     return tpm2_readpublic(ectx, handle, public, NULL, NULL);
 }
 
-tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
+static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
 
     UNUSED(flags);
 
@@ -307,7 +334,7 @@ tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
     return do_hmac_and_output(ectx);
 }
 
-tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
+static tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
     UNUSED(ectx);
 
     if (ctx.input && ctx.input != stdin) {
@@ -316,3 +343,6 @@ tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
 
     return tpm2_session_close(&ctx.hmac_key.object.session);
 }
+
+// Register this tool with tpm2_tool.c
+TPM2_TOOL_REGISTER("hmac", tpm2_tool_onstart, tpm2_tool_onrun, tpm2_tool_onstop, NULL)
