@@ -30,7 +30,9 @@
 #include "files.h"
 #include "log.h"
 #include "tpm2.h"
+#include "tpm2_tool.h"
 #include "tpm2_alg_util.h"
+#include "tpm2_attr_util.h"
 #include "tpm2_auth_util.h"
 #include "tpm2_errata.h"
 #include "tpm2_identity_util.h"
@@ -58,6 +60,7 @@ struct tpm_import_ctx {
     char *policy;
     bool import_tpm; /* Any param that is exclusively used by import tpm object sets this flag */
     TPMI_ALG_PUBLIC key_type;
+    char *cp_hash_path;
 };
 
 static tpm_import_ctx ctx = {
@@ -182,8 +185,23 @@ static tool_rc key_import(ESYS_CONTEXT *ectx, TPM2B_PUBLIC *parent_pub,
     TPMT_SYM_DEF_OBJECT *sym_alg =
             &parent_pub->publicArea.parameters.rsaDetail.symmetric;
 
-    return tpm2_import(ectx, &ctx.parent.object, &enc_sensitive_key, pubkey,
-            &private, encrypted_seed, sym_alg, imported_private);
+    if (!ctx.cp_hash_path) {
+        return tpm2_import(ectx, &ctx.parent.object, &enc_sensitive_key, pubkey,
+            &private, encrypted_seed, sym_alg, imported_private, NULL);
+    }
+
+    TPM2B_DIGEST cp_hash = { .size = 0 };
+    tool_rc rc = tpm2_import(ectx, &ctx.parent.object, &enc_sensitive_key, pubkey,
+            &private, encrypted_seed, sym_alg, imported_private, &cp_hash);
+    if (rc != tool_rc_success) {
+        return rc;
+    }
+
+    bool result = files_save_digest(&cp_hash, ctx.cp_hash_path);
+    if (!result) {
+        rc = tool_rc_general_error;
+    }
+    return rc;
 }
 
 static bool on_option(char key, char *value) {
@@ -238,6 +256,9 @@ static bool on_option(char key, char *value) {
     case 0:
         ctx.auth_key_file = value;
         break;
+    case 1:
+        ctx.cp_hash_path = value;
+        break;
     default:
         LOG_ERR("Invalid option");
         return false;
@@ -246,7 +267,7 @@ static bool on_option(char key, char *value) {
     return true;
 }
 
-bool tpm2_tool_onstart(tpm2_options **opts) {
+static bool tpm2_tool_onstart(tpm2_options **opts) {
 
     const struct option topts[] = {
       { "parent-auth",        required_argument, NULL, 'P'},
@@ -263,6 +284,7 @@ bool tpm2_tool_onstart(tpm2_options **opts) {
       { "policy",             required_argument, NULL, 'L'},
       { "encryption-key",     required_argument, NULL, 'k'},
       { "passin",             required_argument, NULL,  0 },
+      { "cphash",             required_argument, NULL,  1 },
     };
 
     *opts = tpm2_options_new("P:p:G:i:C:U:u:r:a:g:s:L:k:", ARRAY_LEN(topts),
@@ -282,13 +304,6 @@ static tool_rc check_options(void) {
 
     /* Check the tpm import specific options */
     if (ctx.import_tpm) {
-
-        if (!ctx.policy) {
-            LOG_ERR("Expected imported key policy to be specified via \"-L\","
-                    " missing option.");
-            rc = tool_rc_option_error;
-        }
-
         if (!ctx.input_seed_file) {
             LOG_ERR("Expected SymSeed to be specified via \"-s\","
                     " missing option.");
@@ -310,6 +325,11 @@ static tool_rc check_options(void) {
                     " missing option.");
             rc = tool_rc_option_error;
         }
+
+        if (ctx.cp_hash_path) {
+            LOG_WARN("CAUTION CpHash calculation includes parameters that"
+                     "have a derived/random seed!");
+        }
     }
 
     /* Common options */
@@ -411,7 +431,7 @@ static tool_rc openssl_import(ESYS_CONTEXT *ectx) {
      */
     if (ctx.attrs) {
         TPMA_OBJECT *obj_attrs = &public.publicArea.objectAttributes;
-        result = tpm2_util_string_to_uint32(ctx.attrs, obj_attrs);
+        result = tpm2_attr_util_obj_from_optarg(ctx.attrs, obj_attrs);
         if (!result) {
             LOG_ERR("Invalid object attribute, got\"%s\"", ctx.attrs);
             return tool_rc_general_error;
@@ -491,7 +511,7 @@ static tool_rc openssl_import(ESYS_CONTEXT *ectx) {
     TPM2B_PRIVATE *imported_private = NULL;
     tmp_rc = key_import(ectx, parent_pub, &private, &public, &encrypted_seed,
             &imported_private);
-    if (tmp_rc != tool_rc_success) {
+    if (tmp_rc != tool_rc_success || ctx.cp_hash_path) {
         rc = tmp_rc;
         goto keyout;
     }
@@ -598,34 +618,53 @@ static tool_rc tpm_import(ESYS_CONTEXT *ectx) {
         return tool_rc_general_error;
     }
 
-public.publicArea.authPolicy.size = sizeof(public.publicArea.authPolicy.buffer);
-    result = files_load_bytes_from_path(ctx.policy,
+    if (ctx.policy) {
+        public.publicArea.authPolicy.size =
+            sizeof(public.publicArea.authPolicy.buffer);
+        result = files_load_bytes_from_path(ctx.policy,
         public.publicArea.authPolicy.buffer,
-            &public.publicArea.authPolicy.size);
-    if (!result) {
-        return tool_rc_general_error;
+        &public.publicArea.authPolicy.size);
+        if (!result) {
+            LOG_ERR("Failed to copy over the auth policy to the public data");
+            return tool_rc_general_error;
+        }
     }
 
+    if (!ctx.cp_hash_path) {
+        rc = tpm2_import(ectx, &ctx.parent.object, &enc_key, &public, &duplicate,
+            &encrypted_seed, &sym_alg, &imported_private, NULL);
+        if (rc != tool_rc_success) {
+            return rc;
+        }
+
+        assert(imported_private);
+
+        result = files_save_private(imported_private, ctx.private_key_file);
+        free(imported_private);
+        if (!result) {
+            LOG_ERR("Failed to save private key into file \"%s\"",
+                    ctx.private_key_file);
+            return tool_rc_general_error;
+        }
+        return tool_rc_success;
+    }
+
+    TPM2B_DIGEST cp_hash = { .size = 0 };
     rc = tpm2_import(ectx, &ctx.parent.object, &enc_key, &public, &duplicate,
-            &encrypted_seed, &sym_alg, &imported_private);
+            &encrypted_seed, &sym_alg, &imported_private, &cp_hash);
     if (rc != tool_rc_success) {
         return rc;
     }
 
-    assert(imported_private);
-
-    result = files_save_private(imported_private, ctx.private_key_file);
-    free(imported_private);
+    result = files_save_digest(&cp_hash, ctx.cp_hash_path);
     if (!result) {
-        LOG_ERR("Failed to save private key into file \"%s\"",
-                ctx.private_key_file);
-        return tool_rc_general_error;
+        rc = tool_rc_general_error;
     }
+    return rc;
 
-    return tool_rc_success;
 }
 
-tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
+static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
     UNUSED(flags);
 
     tool_rc rc = check_options();
@@ -644,7 +683,7 @@ tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
     return ctx.import_tpm ? tpm_import(ectx) : openssl_import(ectx);
 }
 
-tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
+static tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
     UNUSED(ectx);
 
     if (!ctx.import_tpm) {
@@ -653,3 +692,6 @@ tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
 
     return tpm2_session_close(&ctx.parent.object.session);
 }
+
+// Register this tool with tpm2_tool.c
+TPM2_TOOL_REGISTER("import", tpm2_tool_onstart, tpm2_tool_onrun, tpm2_tool_onstop, NULL)
